@@ -10,7 +10,7 @@
 import re
 import math
 from .helpers import collapse_white_spaces
-from .atomic_types import AtomicTypeABCMeta, AnyAtomicType
+from .atomic_types import AtomicTypeMeta, AnyAtomicType
 
 
 class Float10(float, AnyAtomicType):
@@ -33,6 +33,9 @@ class Float10(float, AnyAtomicType):
             return super().__new__(cls, 0.0)
         return value
 
+    def __hash__(self):
+        return super(Float10, self).__hash__()
+
     def __eq__(self, other):
         if isinstance(other, self.__class__):
             if super(Float10, self).__eq__(other):
@@ -106,7 +109,7 @@ class Float(Float10):
     xsd_version = '1.1'
 
 
-class Integer(int, metaclass=AtomicTypeABCMeta):
+class Integer(int, metaclass=AtomicTypeMeta):
     """A wrapper for emulating xs:integer and limited integer types."""
     name = 'integer'
     pattern = re.compile(r'^[\-+]?[0-9]+$')
