@@ -19,8 +19,9 @@ from ..datatypes import AnyAtomicType, AbstractDateTime, Duration, DayTimeDurati
     xsd10_atomic_types, xsd11_atomic_types, ATOMIC_VALUES
 from ..xpath_context import XPathSchemaContext
 from ..tdop import Parser
-from ..namespaces import XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE, XML_NAMESPACE, \
-    XSD_NAMESPACE, XSD_UNTYPED_ATOMIC, get_namespace, get_expanded_name, split_expanded_name
+from ..namespaces import XML_NAMESPACE, XSD_NAMESPACE, XPATH_FUNCTIONS_NAMESPACE, \
+    XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE, XSD_UNTYPED_ATOMIC, get_namespace, \
+    get_expanded_name, split_expanded_name
 from ..schema_proxy import AbstractSchemaProxy
 from ..xpath_token import XPathToken
 from ..xpath_nodes import XPathNode, TypedElement, AttributeNode, TypedAttribute, \
@@ -104,6 +105,7 @@ class XPath1Parser(Parser):
     schema = None           # XPath 1.0 doesn't have schema bindings
     variable_types = None   # XPath 1.0 doesn't have in-scope variable types
     xsd_version = '1.0'     # Use XSD 1.0 datatypes for default
+    function_namespace = XPATH_FUNCTIONS_NAMESPACE
 
     def __init__(self, namespaces=None, strict=True, *args, **kwargs):
         super(XPath1Parser, self).__init__()
@@ -341,6 +343,8 @@ class XPath1Parser(Parser):
             except (KeyError, AttributeError):
                 return UntypedAtomic('1')
         else:
+            # returns an xs:untypedAtomic value also for element-only types
+            # (that should be None) because it is for static evaluation.
             return UntypedAtomic('1')
 
     def match_sequence_type(self, value, sequence_type, occurrence=None):
@@ -521,9 +525,6 @@ def led(self, left):
 
     self[:] = left, self.parser.expression(90)
     self.value = '{}:{}'.format(self[0].value, self[1].value)
-
-    if self[1].symbol == ':':
-        raise self.wrong_syntax('{!r} is not a QName'.format(self.source))
     return self
 
 
@@ -547,13 +548,7 @@ def select(self, context=None):
     if self[0].value == '*':
         name = '*:%s' % self[1].value
     else:
-        try:
-            namespace = self.get_namespace(self[0].value)
-        except ElementPathKeyError:
-            msg = "prefix {!r} has not been declared".format(self[0].value)
-            raise self.error('XPST0081', msg) from None
-        else:
-            name = '{%s}%s' % (namespace, self[1].value)
+        name = '{%s}%s' % (self.get_namespace(self[0].value), self[1].value)
 
     if context is None:
         yield name
@@ -614,8 +609,7 @@ def nud(self):
 def evaluate(self, context=None):
     if self[1].label.endswith('function'):
         return self[1].evaluate(context)
-    else:
-        return '{%s}%s' % (self[0].value, self[1].value)
+    return [x for x in self.select(context)]
 
 
 @method('{')
@@ -633,7 +627,7 @@ def select(self, context=None):
     elif self.xsd_types is None:
         for item in context.iter_children_or_self():
             if match_attribute_node(item, name):
-                yield item.value
+                yield item
             elif match_element_node(item, name):
                 yield item
     else:
@@ -696,7 +690,7 @@ def select(self, context=None):
                 pass  # '*' wildcard doesn't match document nodes
             elif context.axis == 'attribute':
                 if isinstance(item, (AttributeNode, TypedAttribute)):
-                    yield item.value
+                    yield item
             elif is_element_node(item):
                 yield item
 
@@ -716,13 +710,33 @@ def select(self, context=None):
     if context is None:
         raise self.missing_context()
 
-    for item in context.iter_self():
-        if item is not None:
+    elif isinstance(context, XPathSchemaContext):
+        for item in context.iter_self():
             if is_schema_node(item):
                 self.add_xsd_type(item)
+            elif item is context.root:
+                # item is the schema
+                for xsd_element in item:
+                    self.add_xsd_type(xsd_element)
             yield item
-        elif is_document_node(context.root):
-            yield context.root
+
+    elif self.xsd_types is None:
+        for item in context.iter_self():
+            if item is not None:
+                yield item
+            elif is_document_node(context.root):
+                yield context.root
+
+    else:
+        for item in context.iter_self():
+            if item is not None:
+                if isinstance(item, (TypedAttribute, TypedElement)):
+                    yield item
+                else:
+                    context.item = self.get_typed_node(item)
+                    yield context.item
+            elif is_document_node(context.root):
+                yield context.root
 
 
 @method(nullary('..'))
@@ -783,10 +797,6 @@ def evaluate(self, context=None):
 
 ###
 # Numerical operators
-prefix('+', bp=40)
-prefix('-', bp=70)
-
-
 @method(infix('+', bp=40))
 def evaluate(self, context=None):
     if len(self) == 1:
@@ -798,17 +808,15 @@ def evaluate(self, context=None):
         if op1 is not None:
             try:
                 return op1 + op2
-            except ValueError as err:
-                raise self.error('FORG0001', err) from None
             except TypeError as err:
-                raise self.error('XPTY0004', err)
+                raise self.error('XPTY0004', err) from None
             except OverflowError as err:
                 if isinstance(op1, AbstractDateTime):
-                    raise self.error('FODT0001', err)
+                    raise self.error('FODT0001', err) from None
                 elif isinstance(op1, Duration):
-                    raise self.error('FODT0002', err)
+                    raise self.error('FODT0002', err) from None
                 else:
-                    raise self.error('FOAR0002', err)
+                    raise self.error('FOAR0002', err) from None
 
 
 @method(infix('-', bp=40))
@@ -826,11 +834,18 @@ def evaluate(self, context=None):
                 raise self.error('XPTY0004', err) from None
             except OverflowError as err:
                 if isinstance(op1, AbstractDateTime):
-                    raise self.error('FODT0001', err)
+                    raise self.error('FODT0001', err) from None
                 elif isinstance(op1, Duration):
-                    raise self.error('FODT0002', err)
+                    raise self.error('FODT0002', err) from None
                 else:
-                    raise self.error('FOAR0002', err)
+                    raise self.error('FOAR0002', err) from None
+
+
+@method('+')
+@method('-')
+def nud(self):
+    self[:] = self.parser.expression(rbp=70),
+    return self
 
 
 @method(infix('*', bp=45))
@@ -843,17 +858,17 @@ def evaluate(self, context=None):
                     return op2 * op1
                 return op1 * op2
             except TypeError as err:
-                if isinstance(op1, float):
+                if isinstance(op1, (float, decimal.Decimal)):
                     if math.isnan(op1):
-                        raise self.error('FOCA0005', err) from None
+                        raise self.error('FOCA0005') from None
                     elif math.isinf(op1):
-                        raise self.error('FODT0002', err) from None
+                        raise self.error('FODT0002') from None
 
-                if isinstance(op2, float):
+                if isinstance(op2, (float, decimal.Decimal)):
                     if math.isnan(op2):
-                        raise self.error('FOCA0005', err) from None
+                        raise self.error('FOCA0005') from None
                     elif math.isinf(op2):
-                        raise self.error('FODT0002', err) from None
+                        raise self.error('FODT0002') from None
 
                 raise self.error('XPTY0004', err) from None
             except ValueError as err:
@@ -1063,9 +1078,9 @@ def select(self, context=None):
         raise self.missing_context()
 
     for _ in context.inner_focus_select(self[0]):
-        if self[1].label in ('axis', 'kind test') or self[1].symbol == '..':
-            if not is_xpath_node(context.item):
-                raise self.error('XPTY0020')
+        if (self[1].label in ('axis', 'kind test') or self[1].symbol == '..') \
+                and not is_xpath_node(context.item):
+            raise self.error('XPTY0020')
 
         predicate = [x for x in self[1].select(context.copy())]
         if len(predicate) == 1 and isinstance(predicate[0], NumericProxy):
